ValidationError

Enum ValidationError 

#[non_exhaustive]
pub enum ValidationError {
Show 15 variants ShapeMismatch(Box<ShapeMismatch>), ShapeDataLengthMismatch { expected: usize, actual: usize, }, RankMismatch { expected: usize, actual: usize, }, AxisOutOfBounds { axis: usize, rank: usize, }, DuplicateAxis { axis: usize, role: &'static str, }, AxisRoleConflict { axis: usize, first_role: &'static str, second_role: &'static str, }, InvalidPermutationLength { expected: usize, actual: usize, }, InvalidSliceStep { step: isize, }, InvalidSliceBounds { start: isize, end: isize, axis_len: usize, }, DTypeMismatch { expected: DType, actual: DType, }, InvalidArgument { argument: &'static str, message: String, }, NonContiguousViewAsSlice, ViewOutOfBounds, OverlappingMutableLayout, IntegerOverflow,
}
Structured validation failures owned by the tensor data model.

Examples

use tenferro_tensor_core::{ShapeMismatch, ShapeVec, ValidationError};

let error: ValidationError = ShapeMismatch::ExpectedActual {
    expected: ShapeVec::from_vec(vec![2, 3]),
    actual: ShapeVec::from_vec(vec![6]),
}
.into();
assert!(error.to_string().contains("shape mismatch"));

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


ShapeMismatch(Box<ShapeMismatch>)



ShapeDataLengthMismatch

Fields

expected: usize
actual: usize


RankMismatch

Fields

expected: usize
actual: usize


AxisOutOfBounds

Fields

axis: usize
rank: usize


DuplicateAxis

Fields

axis: usize
role: &'static str


AxisRoleConflict

Fields

axis: usize
first_role: &'static str
second_role: &'static str


InvalidPermutationLength

Fields

expected: usize
actual: usize


InvalidSliceStep

Fields

step: isize


InvalidSliceBounds

Fields

start: isize
end: isize
axis_len: usize


DTypeMismatch

Fields

expected: DType
actual: DType


InvalidArgument

Fields

argument: &'static str
message: String


NonContiguousViewAsSlice



ViewOutOfBounds



OverlappingMutableLayout



IntegerOverflow

Implementations



impl ValidationError

pub fn kind(&self) -> ValidationKind

Return the stable coarse classification for this validation failure.

Examples
use tenferro_tensor_core::{ValidationError, ValidationKind};

let error = ValidationError::AxisOutOfBounds { axis: 3, rank: 2 };
assert_eq!(error.kind(), ValidationKind::AxisOutOfBounds);