Module types

Module types 

Structs

AllocationDomainId
Identity of a backend-owned allocation domain.
AllocationId
Stable physical identity of one backend allocation.
BackendStorageHandle
Backend-owned buffer handle.
CpuDomainId
Caller-stable identity for a CPU execution domain.
DeviceId
Concrete compute device identifier.
DynRank
Dynamic tensor rank marker.
HostReadGuard
Closure-scoped read mapping of a backend allocation.
HostWriteGuard
Closure-scoped write-only mapping of a backend allocation.
Placement
Placement metadata for a tensor buffer.
Rank
Static tensor rank marker.
StridedSliceSpec
One-axis slice specification for typed view slicing.
TensorLayout
Storage-neutral tensor layout metadata.
TensorValue
Owned tensor value with one move-only physical owner and metadata-only layout.
TensorValueViewError
A consuming view transformation failed while retaining its unchanged owner.
TypedTensor
Runtime typed tensor storage with compile-time scalar type and rank metadata.
TypedTensorView
Read-only borrowed view of typed tensor storage with arbitrary strides.
TypedTensorViewMut
Mutable borrowed view of typed tensor storage with arbitrary strides.

Enums

DType
Runtime scalar dtype tag.
DeviceKind
Compute device family.
GpuBackendKind
GPU backend family used by placement metadata.
HostAccessError
Typed failure returned by guarded backend host access.
MemoryKind
Memory location for tensor storage.
StorageBuffer
Tensor storage.
Tensor
Dynamic tensor enum over the supported scalar types.
TensorRead
Read-only tensor input accepted by synchronous eager kernels.
TensorStorageRef
Borrowed tensor buffer reference used by read-only typed views.
TensorStorageRefMut
Borrowed tensor buffer reference used by mutable typed views.
TensorView
Dynamic read-only borrowed tensor view.
TensorViewMut
Dynamic mutable borrowed tensor view.
TensorWrite
Mutable tensor output accepted by synchronous eager kernels.
TypedTensorWrite
Mutable typed tensor output accepted by synchronous eager kernels.

Traits

BackendStorage
Opaque backend-owned tensor buffer.
SharedTensorAllocationDomain
Backend-neutral owner of one shared tensor allocation domain.
TensorRank
Rank contract for tensor metadata shapes and strides.
TensorScalar
Sealed trait for scalar types that can be stored in a Tensor.

Functions

col_major_strides
Column-major strides derived from a shape.

Type Aliases

TypedTensorViewMutSplit
Pair of mutable tensor views returned by disjoint multi-slice operations.