TensorBufferRefMut

Enum TensorBufferRefMut 

pub enum TensorBufferRefMut<'a, T> {
    Host(&'a mut [T]),
    Backend(Arc<dyn BackendBuffer<T>>),
}
Borrowed tensor buffer reference used by mutable typed views.

Backend buffers can be represented for residency metadata, but this crate does not expose host mutation for backend-native allocations.

Examples

use tenferro_tensor::TensorBufferRefMut;

let mut data = [1_i32, 2];
let buffer = TensorBufferRefMut::Host(&mut data);
assert_eq!(buffer.len(), 2);

Variants



Host(&'a mut [T])



Backend(Arc<dyn BackendBuffer<T>>)

Implementations

Source

impl<T: 'static> TensorBufferRefMut<'_, T>

pub fn len(&self) -> usize

Return the logical length of the backing allocation.

Examples
use tenferro_tensor::TensorBufferRefMut;

let mut data = [1_i32, 2, 3];
assert_eq!(TensorBufferRefMut::Host(&mut data).len(), 3);
pub fn is_empty(&self) -> bool

Return whether the backing allocation is empty.

Examples
use tenferro_tensor::TensorBufferRefMut;

let mut data: [f64; 0] = [];
assert!(TensorBufferRefMut::Host(&mut data).is_empty());