pub struct AllocationId(/* private fields */);Expand description
Stable physical identity of one backend allocation.
Examples
use tenferro_tensor::AllocationId;
assert_eq!(AllocationId::from_backend_id(7), AllocationId::from_backend_id(7));Implementations
Sourceimpl AllocationId
impl AllocationId
Sourcepub const fn from_backend_id(id: u64) -> Self
pub const fn from_backend_id(id: u64) -> Self
Wrap an allocation identity supplied by the owning backend.
Examples
use tenferro_tensor::AllocationId;
let id = AllocationId::from_backend_id(3);
assert_eq!(id, AllocationId::from_backend_id(3));Trait Implementations
Sourceimpl Clone for AllocationId
impl Clone for AllocationId
Sourcefn clone(&self) -> AllocationId
fn clone(&self) -> AllocationId
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for AllocationId
impl Debug for AllocationId
Sourceimpl Hash for AllocationId
impl Hash for AllocationId
Sourceimpl Ord for AllocationId
impl Ord for AllocationId
Sourcefn cmp(&self, other: &AllocationId) -> Ordering
fn cmp(&self, other: &AllocationId) -> Ordering
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialEq for AllocationId
impl PartialEq for AllocationId
Sourcefn eq(&self, other: &AllocationId) -> bool
fn eq(&self, other: &AllocationId) -> bool
Tests for
self and other values to be equal, and is used by ==.Sourceimpl PartialOrd for AllocationId
impl PartialOrd for AllocationId
impl Copy for AllocationId
impl Eq for AllocationId
impl StructuralPartialEq for AllocationId