Runtime error types for tensor execution.
Examples
let error = tenferro_tensor::Error::shape_mismatch("add", [2], [3]);
assert!(matches!(
error,
tenferro_tensor::Error::Validation { op: "add", .. }
));Structs
- Reinterpret
Error - Owns the original tensor when a consuming representation reinterpretation cannot publish its checked descriptor.
Enums
- Error
- Runtime failures produced by tensor execution backends and helpers.