pub enum Error {
AxisOutOfBounds {
op: &'static str,
axis: usize,
rank: usize,
},
DuplicateAxis {
op: &'static str,
axis: usize,
role: &'static str,
},
AxisRoleConflict {
op: &'static str,
axis: usize,
first_role: &'static str,
second_role: &'static str,
},
ShapeMismatch {
op: &'static str,
lhs: Vec<usize>,
rhs: Vec<usize>,
},
RankMismatch {
op: &'static str,
expected: usize,
actual: usize,
},
DTypeMismatch {
op: &'static str,
lhs: DType,
rhs: DType,
},
UnsupportedDTypeConversion {
op: &'static str,
from: DType,
to: DType,
message: String,
},
InvalidConfig {
op: &'static str,
message: String,
},
BackendFailure {
op: &'static str,
message: String,
},
MissingValue {
slot: usize,
},
}Expand description
Runtime failures produced by tensor execution backends and helpers.
Examples
use tenferro_tensor::Error;
let err = Error::MissingValue { slot: 3 };Variants
AxisOutOfBounds
DuplicateAxis
AxisRoleConflict
ShapeMismatch
RankMismatch
DTypeMismatch
UnsupportedDTypeConversion
InvalidConfig
BackendFailure
MissingValue
Implementations
Sourceimpl Error
impl Error
Sourcepub fn backend_failure(op: &'static str, message: impl Display) -> Self
pub fn backend_failure(op: &'static str, message: impl Display) -> Self
Construct a backend failure error while preserving the operation name.
Examples
use tenferro_tensor::Error;
let err = Error::backend_failure("matmul", "backend rejected launch");
assert!(matches!(
err,
Error::BackendFailure {
op: "matmul",
ref message,
} if message == "backend rejected launch"
));Trait Implementations
Sourceimpl Error for Error
impl Error for Error
1.30.0 · Sourcefn source(&self) -> Option<&(dyn Error + 'static)>
fn source(&self) -> Option<&(dyn Error + 'static)>
Returns the lower-level source of this error, if any. Read more
1.0.0 · Sourcefn description(&self) -> &str
fn description(&self) -> &str
👎Deprecated since 1.42.0:
use the Display impl or to_string()
impl Eq for Error
impl StructuralPartialEq for Error