pub struct BackendStorageHandle<T> { /* private fields */ }Expand description
Backend-owned buffer handle.
BackendStorageHandle::new creates an empty opaque handle. Use
BackendStorageHandle::new_with_len when test or adapter code needs to model a
non-empty backend allocation.
Examples
use tenferro_tensor::BackendStorageHandle;
let handle = BackendStorageHandle::<f64>::new(7);Implementations
Sourceimpl<T> BackendStorageHandle<T>
impl<T> BackendStorageHandle<T>
Sourcepub fn new(id: u64) -> Self
pub fn new(id: u64) -> Self
Create a new backend buffer handle.
Examples
use tenferro_tensor::BackendStorageHandle;
let handle = BackendStorageHandle::<f64>::new(1);
assert_eq!(tenferro_tensor::BackendStorage::len(&handle), 0);Sourcepub fn new_with_len(id: u64, len: usize) -> Self
pub fn new_with_len(id: u64, len: usize) -> Self
Create a new backend buffer handle with a logical element count.
Examples
use tenferro_tensor::{BackendStorage, BackendStorageHandle};
let handle = BackendStorageHandle::<f64>::new_with_len(1, 4);
assert_eq!(BackendStorage::len(&handle), 4);Trait Implementations
Sourceimpl<T: Send + Sync + 'static> BackendStorage<T> for BackendStorageHandle<T>
impl<T: Send + Sync + 'static> BackendStorage<T> for BackendStorageHandle<T>
Sourcefn backend_family(&self) -> &'static str
fn backend_family(&self) -> &'static str
Stable backend family identifier.
Sourcefn allocation_domain(&self) -> Option<AllocationDomainId>
fn allocation_domain(&self) -> Option<AllocationDomainId>
Return the shared-allocation domain, when this buffer belongs to one.
Sourcefn allocation_id(&self) -> Option<AllocationId>
fn allocation_id(&self) -> Option<AllocationId>
Return the stable physical allocation identity, when available.
Sourcefn as_any(&self) -> &dyn Any
fn as_any(&self) -> &dyn Any
Type-erased access for the backend crate that owns the concrete handle.
Sourcefn map_read(&self) -> Result<HostReadGuard<'_, T>, HostAccessError>
fn map_read(&self) -> Result<HostReadGuard<'_, T>, HostAccessError>
Map the allocation for closure-scoped host reads. Read more
Sourcefn map_write(&mut self) -> Result<HostWriteGuard<'_, T>, HostAccessError>
fn map_write(&mut self) -> Result<HostWriteGuard<'_, T>, HostAccessError>
Map the allocation for closure-scoped host writes. Read more