BufferHandle

Struct BufferHandle 

pub struct BufferHandle<T> { /* private fields */ }
Backend-owned buffer handle.

BufferHandle::new creates an empty opaque handle. Use BufferHandle::new_with_len when test or adapter code needs to model a non-empty backend allocation.

Examples

use tenferro_tensor::BufferHandle;

let handle = BufferHandle::<f64>::new(7);

Implementations

Source

impl<T> BufferHandle<T>

pub fn new(id: u64) -> Self

Create a new backend buffer handle.

Examples
use tenferro_tensor::BufferHandle;

let handle = BufferHandle::<f64>::new(1);
assert_eq!(tenferro_tensor::BackendBuffer::len(&handle), 0);
pub fn new_with_len(id: u64, len: usize) -> Self

Create a new backend buffer handle with a logical element count.

Examples
use tenferro_tensor::{BackendBuffer, BufferHandle};

let handle = BufferHandle::<f64>::new_with_len(1, 4);
assert_eq!(BackendBuffer::len(&handle), 4);