pub struct TypedTensor<T> {
pub buffer: Buffer<T>,
pub shape: Vec<usize>,
pub placement: Placement,
}Expand description
Fields
buffer: Buffer<T>shape: Vec<usize>placement: PlacementImplementations
Sourceimpl<T: TensorScalar> TypedTensor<T>
impl<T: TensorScalar> TypedTensor<T>
Sourcepub fn svd(
&self,
ctx: &mut impl TensorBackend,
) -> Result<(Self, TypedTensor<T::Real>, Self)>
pub fn svd( &self, ctx: &mut impl TensorBackend, ) -> Result<(Self, TypedTensor<T::Real>, Self)>
Singular value decomposition: A = U diag(S) Vt.
Returns (U, S, Vt) using the thin/economy SVD.
For complex inputs, this wrapper expects the backend to return the
singular values as TypedTensor<T::Real>. If the backend still returns
a complex tensor for S, this method returns Error::DTypeMismatch.
Examples
use tenferro_tensor::{cpu::CpuBackend, TensorBackend, TypedTensor};
let mut ctx = CpuBackend::new();
let a = TypedTensor::<f64>::from_vec(vec![2, 2], vec![1.0, 0.0, 0.0, 2.0]);
let (u, s, vt) = a.svd(&mut ctx).unwrap();
assert_eq!(u.shape, vec![2, 2]);
assert_eq!(s.shape, vec![2]);
assert_eq!(vt.shape, vec![2, 2]);Sourcepub fn qr(&self, ctx: &mut impl TensorBackend) -> Result<(Self, Self)>
pub fn qr(&self, ctx: &mut impl TensorBackend) -> Result<(Self, Self)>
QR decomposition: A = Q R.
Returns (Q, R) using the thin/economy QR decomposition.
Examples
use tenferro_tensor::{cpu::CpuBackend, TensorBackend, TypedTensor};
let mut ctx = CpuBackend::new();
let a = TypedTensor::<f64>::from_vec(vec![2, 2], vec![1.0, 2.0, 3.0, 4.0]);
let (q, r) = a.qr(&mut ctx).unwrap();
assert_eq!(q.shape, vec![2, 2]);
assert_eq!(r.shape, vec![2, 2]);Sourcepub fn cholesky(&self, ctx: &mut impl TensorBackend) -> Result<Self>
pub fn cholesky(&self, ctx: &mut impl TensorBackend) -> Result<Self>
Cholesky factorization: A = L L^T or A = L L^H.
Returns the lower-triangular factor L.
Examples
use tenferro_tensor::{cpu::CpuBackend, TensorBackend, TypedTensor};
let mut ctx = CpuBackend::new();
let a = TypedTensor::<f64>::from_vec(vec![2, 2], vec![4.0, 1.0, 1.0, 3.0]);
let l = a.cholesky(&mut ctx).unwrap();
assert_eq!(l.shape, vec![2, 2]);Sourcepub fn eigh(
&self,
ctx: &mut impl TensorBackend,
) -> Result<(TypedTensor<T::Real>, Self)>
pub fn eigh( &self, ctx: &mut impl TensorBackend, ) -> Result<(TypedTensor<T::Real>, Self)>
Symmetric or Hermitian eigendecomposition: A = V diag(W) V^T.
Returns (eigenvalues, eigenvectors).
For complex inputs, this wrapper expects the backend to return the
eigenvalues as TypedTensor<T::Real>. If the backend still returns a
complex tensor for W, this method returns Error::DTypeMismatch.
Examples
use tenferro_tensor::{cpu::CpuBackend, TensorBackend, TypedTensor};
let mut ctx = CpuBackend::new();
let a = TypedTensor::<f64>::from_vec(vec![2, 2], vec![4.0, 1.0, 1.0, 3.0]);
let (w, v) = a.eigh(&mut ctx).unwrap();
assert_eq!(w.shape, vec![2]);
assert_eq!(v.shape, vec![2, 2]);Sourceimpl<T: Clone + Zero> TypedTensor<T>
impl<T: Clone + Zero> TypedTensor<T>
Sourceimpl<T: Clone> TypedTensor<T>
impl<T: Clone> TypedTensor<T>
Sourcepub fn n_elements(&self) -> usize
pub fn n_elements(&self) -> usize
Sourcepub fn as_slice(&self) -> &[T]
pub fn as_slice(&self) -> &[T]
View the tensor data as a flat slice.
This is an alias for host_data() for API consistency with
Tensor::as_slice.
Examples
use tenferro_tensor::TypedTensor;
let t = TypedTensor::<f64>::from_vec(vec![2], vec![1.0, 2.0]);
assert_eq!(t.as_slice(), &[1.0, 2.0]);Sourcepub fn host_data_mut(&mut self) -> &mut [T]
pub fn host_data_mut(&mut self) -> &mut [T]
Sourcepub fn linear_offset(&self, indices: &[usize]) -> usize
pub fn linear_offset(&self, indices: &[usize]) -> usize